!-------------------------------------------------------------------------
!  add wave heating: (to ions) X.Y.Wu on 03/05/98
!  Documented and modified: Jiannan Tu, Feb 13, 2003
!
!   Input:  v     - ion (O+ or H+) perpendicular velocity (in Re/s)
!           dpp   - ion heating rate in ergs/s
!           dt    - time interval (s)
!           m     - ion (O+ or H+) mass (kg)
!
!   Output: v     - ion perp velocity after heating (in Re/s)
!
!  Rewitten by Jiannan Tu, Feb 13, 2003. JNT 1/25/10 Rev
!-------------------------------------------------------------------------
subroutine waveheat(ijk,s,v,dpp,dt,m)
    use gskparm

    real(8) dpp(nc)

    real(8) s,v,dt,m,w,theta,sig,dw,w1,Re,pi2,r2,gau,dppi,bb,ran2
    integer ijk

    data r2/1.4142136d0/,pi2/6.283185308d0/,Re/6371.2d+3/

! interpolate heating rate to particle location
    bb=(s-sg(ijk))*ods(ijk)
    dppi=(dpp(ijk+1)-dpp(ijk))*bb+dpp(ijk)

    if(dppi.le.0.0d0) return

    w=0.5d7*m*(v*Re)**2

    theta=pi2*ran2(iseed)
    sig=r2*dppi*dt
    dw=gau(sig)
    w1=w+dw+2.*sqrt(w)*sqrt(dw)*cos(theta)

    v=sqrt(2.0d-7*w1/m)/Re

end subroutine waveheat

!---------------------------------------------------------------------
!  Calculate the fixed ion cyclotron wave - induced anomalous ion
!  perpendicular heating rate (erg/s) profile.    X.Y.Wu 03/08/98
!  Crew	(1990) JGR, 3969, eqs.(17, 19-22)
!  Brown (1995) JGR,17500, eqs.(2,3)
!
!  Input:
!     e0:       spectrum density at fco+ = 6.5 Hz at 1 Re altitude
!               in (mV)^2 m^-2 Hz^-1
!     alpha:    power-law wave spectral index
!     rh0(1):   lower cutoff altitude
!     rh0(2):   upper cutoff altitude
!     rhscl(1): scale height for lower altitude cutoff
!     rhscl(2): scale height for upper altitude cutoff
!
!  Output:
!     dppo(1:nc)  perpendicular O+ heating rate (ergs/s)
!     dpph(1:nc)  perpendicular H+ heating rate (ergs/s)
!---------------------------------------------------------------------
subroutine waveh_rate(e0,dppo,dpph,dpphe)
    use gskparm

    integer i

    real(8) e0,dppo(nc),dpph(nc),dpphe(nc)
    real(8) con,r0,e0v,a_O,a_h,a_he,coef,fac

    data r0/2.0d0/,a_o/15.9994d0/,a_h/1.0008d0/,a_he/4.00012d0/
!      data e/1.6022d-19/,amu/1.66053873d-27/,eta/0.125/
!      con=(0.5d0*eta*e/amu)*(e*1.e7)

    con=1.03743d-5

!... convert spectrum density to V^2 m^-2 Hz^-1
    e0v=e0*1.0d-6
    coef = con*e0v

    rh0(1)=rh0(1)+1.
    rh0(2)=rh0(2)+1.

    do i=1,num
        fac = (rr(i)/r0)**(3.0d0*alpha)
!... This shuts off power in O and H above rh0
        if (rr(i).gt.rh0(2)) fac=fac*exp(-(rr(i)-rh0(2))/rhscl(2))
        if (rr(i).lt.rh0(1)) fac=fac*exp((rr(i)-rh0(1))/rhscl(1))
        dppo(i) = coef*fac/a_o
        dpph(i) = coef*fac*(a_h/a_o)**alpha
        dpphe(i) = coef*fac*(a_he/a_o)**alpha

!... Make correction so dT is right ?
!jntu        dppo(i)=dppo(i)/1.24
!jntu        dpph(i)=dpph(i)/1.24
    enddo

end subroutine waveh_rate
