!     
! File:   moments_mp.f90
! Author: wangy
!
! Created on August 6, 2010, 12:22 PM
! Modified on Jun.8,2011 

!------------------------------------------------------------------------
!  Calculate ion (O+ or H+) moments (density, para velocity, para and
!  perp temperature, para and perp heat flow)
!
!    Input:
!       vol      - volume of grid cells (cm^3)
!       OHgen    - array of location (in Re), para and perp velocity
!                  of simulation particles
!       imax     - total number of simulation particles (O+ or H+)
!       rep_s    - number of real(8) ion represented by a simulation
!                  particle (O+ or H+)
!       ns       - = 0 for O+; = 1 for H+
!
!    Output:
!       den_s  - ion (O+ or H+) density (cm^-3)
!       vel_s  - ion parallel velocity (m/s)
!       tpr_s  - ion paralel temperature (K)
!       tpp_s  - ion perpendicular temperature (K)
!       qpr_s  - ion parallel heat flow (J m^-2 s^-1)
!       qpp_s  - ion perpendicular heat flow (J m^-2 s^-1)
!
!  Jiannan Tu, Feb 14, 2003; 5/28/2010
!------------------------------------------------------------------------
subroutine moments_s(OHgen,imax,rep_s,den_s,vel_s,tpr_s,tpp_s,qpr_s,qpp_s,ns)
    use gskparm

    integer ns,imax,i,j,num1

    real(8), dimension(np,3) :: OHgen
    real(8) :: den_s,vel_s,tpr_s,tpp_s,qpr_s,qpp_s
    real(8) :: vel,tpr,tpp,qpr,qpp,den
    real(8), dimension(0:2) :: m,mkb
    real(8) :: rep_s,Re,Re2,Re3
    real(8) :: s,vs,vs2,vp2,vs3,vp3

    integer rank,ierr,status
    common /mpi/rank,ierr,status


    data Re/6371.2e+3/
    data m/2.65676233d-26,1.67382304d-27,6.646476162d-27/
    data mkb/1.9242837d-3,1.2123442d-4,4.813845268d-4/

    Re2=Re*Re
    Re3=Re2*Re
    num1=num+1

    den=0.0d0
    vel=0.0d0
    tpr=0.0d0
    tpp=0.0d0
    qpr=0.0d0
    qpp=0.0d0


! den(ijk) - number of super ptcls assigned to grid ijk
    do j=1,imax
        s=OHgen(j,1)
        if (s.lt.sg(1).or.s.ge.sg(num1)) cycle

        den=den+1
        
        vs=OHgen(j,2)
        vel=vel+vs

        vp2=OHgen(j,3)**2
        tpp=tpp+vp2
    enddo

! average or bulk velocity parallel to magnetic field
    if(den.gt.0.d0) then
        vel_s=vel/den
    else
        vel_s=0.0d0
    endif

    do j=1,imax
        s=OHgen(j,1)
        if (s.lt.sg(1).or.s.ge.sg(num+1)) cycle

        vs=OHgen(j,2)
        vs2=(vs-vel_s)**2
        vs3=vs2*(vs-vel_s)

        tpr=tpr+vs2

        qpr=qpr+vs3

        vp2=OHgen(j,3)**2
        vp3=vp2*(vs-vel_s)

        qpp=qpp+vp3

    enddo

!  ion density in cm^-3; temp in J; velocity in m/s; heat flow in J m^-2 /s
    den_s=1.0d-6*(rep_s*den)/c_vol(rank+1)
    vel_s=vel_s*Re
    if(den.gt.0.d0) then
        tpr_s=tpr/den*mkb(ns)*Re2
        tpp_s=0.5*tpp/den*mkb(ns)*Re2
        qpr_s=1.0d6*m(ns)*den_s*qpr/den*Re3
        qpp_s=0.5d6*m(ns)*den_s*qpp/den*Re3
    else
        tpr_s=0.0d0
        tpp_s=0.0d0
        qpr_s=0.0d0
        qpp_s=0.0d0
    endif

end subroutine moments_s
!------------------------------------------------------------------------
!  Sum ion (O+ or H+) moments between two outputs
!
!    Input: moments at current time
!       num    - number of grid cells
!       den_s  - ion (O+ or H+) density (cm^-3)
!       vel_s  - ion parallel velocity (m/s)
!       tpr_s  - ion paralel temperature (K)
!       tpp_s  - ion perpendicular temperature (K)
!       qpr_s  - ion parallel heat flow (J m^-2 s^-1)
!       qpp_s  - ion perpendicular heat flow (J m^-2 s^-1)
!
!    Output: sum of moments between two outputs
!       dens  - ion (O+ or H+) density (cm^-3)
!       vels  - ion parallel velocity (m/s)
!       tprs  - ion paralel temperature (K)
!       tpps  - ion perpendicular temperature (K)
!       qprs  - ion parallel heat flow (J m^-3 s^-1)
!       qpps  - ion perpendicular heat flow (J m^-3 s^-1)
!
!  Jiannan Tu, March 17, 2003
!------------------------------------------------------------------------
subroutine sum_moments(den_s,vel_s,tpr_s,tpp_s,qpr_s,qpp_s,dens, &
                       vels,tprs,tpps,qprs,qpps)

      use gskparm

      integer i

      real*8 den_s(nc),vel_s(nc),tpr_s(nc),tpp_s(nc),qpr_s(nc),qpp_s(nc)
      real(8) dens(nc),vels(nc),tprs(nc),tpps(nc),qprs(nc),qpps(nc)

!  ion density in cm^-3; velocity in m/s; heat flow in J m /s
      do i=1,num1
         dens(i)=dens(i)+den_s(i)
         vels(i)=vels(i)+vel_s(i)
         tprs(i)=tprs(i)+tpr_s(i)
         tpps(i)=tpps(i)+tpp_s(i)
         qprs(i)=qprs(i)+qpr_s(i)
         qpps(i)=qpps(i)+qpp_s(i)
      enddo

end subroutine sum_moments

